==> firestore_search-0.0.2.tar.gz: lib/firestore_search.dart <==
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'firestore_service.dart';

class FirestoreSearchScaffold extends StatefulWidget {
  /// Creates a scaffold with a search AppBar and integrated cloud_firestore search.
  ///
  /// You can set the scaffold body using the [scaffoldBody] widget
  ///
  /// You can add a bottom widget to search AppBar using the [appBarBottom] widget
  ///
  /// [firestoreCollectionName] , [dataListFromSnapshot] are required

  final Widget scaffoldBody;

  final PreferredSizeWidget appBarBottom;
  final Color appBarBackgroundColor,
      backButtonColor,
      clearSearchButtonColor,
      searchBackgroundColor,
      searchTextColor,
      searchTextHintColor,
      scaffoldBackgroundColor,
      searchBodyBackgroundColor;

  final String firestoreCollectionName;
  final List Function(QuerySnapshot) dataListFromSnapshot;
  final Widget Function(BuildContext, AsyncSnapshot) builder;
  final int limitOfRetrievedData;

  const FirestoreSearchScaffold({
    this.appBarBottom,
    this.scaffoldBody = const Center(child: Text('Add a scaffold body')),
    this.appBarBackgroundColor = Colors.white,
    this.backButtonColor = Colors.blue,
    this.clearSearchButtonColor = Colors.blue,
    this.searchBackgroundColor,
    this.searchTextColor,
    this.searchTextHintColor,
    this.scaffoldBackgroundColor = Colors.white,
    this.searchBodyBackgroundColor = Colors.white,

    /// Name of the cloud_firestore collection you
    /// want to search data from
    @required this.firestoreCollectionName,

    /// This function takes QuerySnapshot as an argument an returns the object of your dataMode
    /// See example of such a function here
    @required this.dataListFromSnapshot,

    /// Refers to the [builder] parameter of StreamBuilder used to
    /// retrieve search results from cloud_firestore
    /// Use this function display the search results retrieved from cloud_firestore
    this.builder,
    this.limitOfRetrievedData = 10,
  })  : //Firestore parameters assertions
        assert(limitOfRetrievedData >= 1 && limitOfRetrievedData <= 30,
            'limitOfRetrievedData should be between 1 and 30.\n'),
        assert(firestoreCollectionName != null,
            'firestoreCollectionName is required.\n'),
        assert(dataListFromSnapshot != null,
            'dataListFromSnapshot is required, this function converts QuerySnapshot from firestore to a list of your custom data model.\n');

  @override
  _FirestoreSearchScaffoldState createState() =>
      _FirestoreSearchScaffoldState();
}

class _FirestoreSearchScaffoldState extends State<FirestoreSearchScaffold> {
  TextEditingController searchQueryController = TextEditingController();
  bool isSearching = false;
  String searchQuery = "";
  FocusNode searchFocusNode = FocusNode();

  @override
  void initState() {
    // TODO: implement initState
    searchFocusNode.addListener(() {
      if (searchFocusNode.hasFocus) {
        setState(() {
          isSearching = true;
        });
      }
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: widget.scaffoldBackgroundColor,
        appBar: AppBar(
          backgroundColor: widget.appBarBackgroundColor,
          centerTitle: true,
          automaticallyImplyLeading: true,
          titleSpacing: 0.0,
          title: Row(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              if (isSearching)
                BackButton(
                  color: widget.backButtonColor,
                  onPressed: () {
                    setState(() {
                      isSearching = false;
                      searchFocusNode.unfocus();
                      clearSearchQuery();
                    });
                  },
                ),
              Expanded(
                child: Container(
                  alignment: Alignment.center,
                  height: 40.0,
                  margin: isSearching
                      ? const EdgeInsets.only(
                          bottom: 3.5, top: 3.5, right: 10.0, left: 0.0)
                      : const EdgeInsets.only(
                          bottom: 3.5, top: 3.5, right: 10.0, left: 10.0),
                  padding: const EdgeInsets.symmetric(
                      horizontal: 8.0, vertical: 0.0),
                  decoration: BoxDecoration(
                    borderRadius: BorderRadius.circular(6.0),
                    color: widget?.searchBackgroundColor ??
                        Colors.blueGrey.withOpacity(.2),
                  ),
                  child: TextField(
                    controller: searchQueryController,
                    focusNode: searchFocusNode,
                    autofocus: false,
                    decoration: InputDecoration(
                      hintText: "Search...",
                      border: InputBorder.none,
                      hintStyle: widget?.searchTextHintColor ??
                          TextStyle(color: Colors.black.withOpacity(.6)),
                      suffixIcon: searchQueryController.text.isNotEmpty
                          ? IconButton(
                              alignment: Alignment.centerRight,
                              color: widget.clearSearchButtonColor,
                              icon: const Icon(Icons.clear),
                              onPressed: clearSearchQuery,
                            )
                          : SizedBox(
                              height: 0.0,
                              width: 0.0,
                            ),
                    ),
                    textAlignVertical: TextAlignVertical.center,
                    textInputAction: TextInputAction.search,
                    style: TextStyle(
                        color: widget?.searchTextColor ??
                            Colors.black.withOpacity(.8),
                        fontSize: 16.0),
                    onChanged: (query) => updateSearchQuery(query),
                  ),
                ),
              ),
            ],
          ),
          bottom: isSearching ? null : widget.appBarBottom,
        ),
        body: Stack(
          children: [
            widget.scaffoldBody,
            isSearching
                ? Container(
                    width: MediaQuery.of(context).size.width,
                    height: MediaQuery.of(context).size.height,
                    color: widget.searchBodyBackgroundColor,
                    child: StreamBuilder<List>(
                        stream: FirestoreServicePackage(
                                collectionName: widget.firestoreCollectionName,
                                dataListFromSnapshot:
                                    widget.dataListFromSnapshot,
                                limitOfRetrievedData:
                                    widget.limitOfRetrievedData)
                            .searchData(searchQuery),
                        builder: widget.builder),
                  )
                : SizedBox(
                    height: 0.0,
                    width: 0.0,
                  ),
          ],
        ));
  }

  void updateSearchQuery(String newQuery) {
    setState(() {
      searchQuery = newQuery;
    });
  }

  void stopSearching() {
    clearSearchQuery();
    setState(() {
      isSearching = false;
    });
  }

  void clearSearchQuery() {
    setState(() {
      searchQueryController.clear();
      updateSearchQuery("");
    });
  }

  @override
  void dispose() {
    // TODO: implement dispose
    searchQueryController.dispose();
    super.dispose();
  }
}

==> firestore_search-0.0.6.tar.gz: lib/firestore_search.dart <==
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'firestore_service.dart';

class FirestoreSearchScaffold extends StatefulWidget {
  /// Creates a scaffold with a search AppBar and integrated cloud_firestore search.
  ///
  /// You can set the scaffold body using the [scaffoldBody] widget
  ///
  /// You can add a bottom widget to search AppBar using the [appBarBottom] widget
  ///
  /// [firestoreCollectionName] , [dataListFromSnapshot] are required

  final Widget scaffoldBody;

  final PreferredSizeWidget appBarBottom;
  final Color appBarBackgroundColor,
      backButtonColor,
      clearSearchButtonColor,
      searchBackgroundColor,
      searchTextColor,
      searchTextHintColor,
      scaffoldBackgroundColor,
      searchBodyBackgroundColor;

  final String firestoreCollectionName, searchBy;
  final List Function(QuerySnapshot) dataListFromSnapshot;
  final Widget Function(BuildContext, AsyncSnapshot) builder;
  final int limitOfRetrievedData;

  const FirestoreSearchScaffold({
    this.appBarBottom,
    this.scaffoldBody = const Center(child: Text('Add a scaffold body')),
    this.appBarBackgroundColor = Colors.white,
    this.backButtonColor = Colors.blue,
    this.clearSearchButtonColor = Colors.blue,
    this.searchBackgroundColor,
    this.searchTextColor,
    this.searchTextHintColor,
    this.scaffoldBackgroundColor = Colors.white,
    this.searchBodyBackgroundColor = Colors.white,

    /// Name of the cloud_firestore collection you
    /// want to search data from
    @required this.firestoreCollectionName,
    @required this.searchBy,

    /// This function takes QuerySnapshot as an argument an returns the object of your dataMode
    /// See example of such a function here
    @required this.dataListFromSnapshot,

    /// Refers to the [builder] parameter of StreamBuilder used to
    /// retrieve search results from cloud_firestore
    /// Use this function display the search results retrieved from cloud_firestore
    this.builder,
    this.limitOfRetrievedData = 10,
  })  : //Firestore parameters assertions
        assert(limitOfRetrievedData >= 1 && limitOfRetrievedData <= 30,
            'limitOfRetrievedData should be between 1 and 30.\n'),
        assert(firestoreCollectionName != null,
            'firestoreCollectionName is required.\n'),
        assert(dataListFromSnapshot != null,
            'dataListFromSnapshot is required, this function converts QuerySnapshot from firestore to a list of your custom data model.\n');

  @override
  _FirestoreSearchScaffoldState createState() =>
      _FirestoreSearchScaffoldState();
}

class _FirestoreSearchScaffoldState extends State<FirestoreSearchScaffold> {
  TextEditingController searchQueryController = TextEditingController();
  bool isSearching = false;
  String searchQuery = "";
  FocusNode searchFocusNode = FocusNode();

  @override
  void initState() {
    // TODO: implement initState
    searchFocusNode.addListener(() {
      if (searchFocusNode.hasFocus) {
        setState(() {
          isSearching = true;
        });
      }
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: widget.scaffoldBackgroundColor,
        appBar: AppBar(
          backgroundColor: widget.appBarBackgroundColor,
          centerTitle: true,
          automaticallyImplyLeading: true,
          titleSpacing: 0.0,
          title: Row(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              if (isSearching)
                BackButton(
                  color: widget.backButtonColor,
                  onPressed: () {
                    setState(() {
                      isSearching = false;
                      searchFocusNode.unfocus();
                      clearSearchQuery();
                    });
                  },
                ),
              Expanded(
                child: Container(
                  alignment: Alignment.center,
                  height: 40.0,
                  margin: isSearching
                      ? const EdgeInsets.only(
                          bottom: 3.5, top: 3.5, right: 10.0, left: 0.0)
                      : const EdgeInsets.only(
                          bottom: 3.5, top: 3.5, right: 10.0, left: 10.0),
                  padding: const EdgeInsets.symmetric(
                      horizontal: 8.0, vertical: 0.0),
                  decoration: BoxDecoration(
                    borderRadius: BorderRadius.circular(6.0),
                    color: widget?.searchBackgroundColor ??
                        Colors.blueGrey.withOpacity(.2),
                  ),
                  child: TextField(
                    controller: searchQueryController,
                    focusNode: searchFocusNode,
                    autofocus: false,
                    decoration: InputDecoration(
                      hintText: "Search...",
                      border: InputBorder.none,
                      hintStyle: widget?.searchTextHintColor ??
                          TextStyle(color: Colors.black.withOpacity(.6)),
                      suffixIcon: searchQueryController.text.isNotEmpty
                          ? IconButton(
                              alignment: Alignment.centerRight,
                              color: widget.clearSearchButtonColor,
                              icon: const Icon(Icons.clear),
                              onPressed: clearSearchQuery,
                            )
                          : SizedBox(
                              height: 0.0,
                              width: 0.0,
                            ),
                    ),
                    textAlignVertical: TextAlignVertical.center,
                    textInputAction: TextInputAction.search,
                    style: TextStyle(
                        color: widget?.searchTextColor ??
                            Colors.black.withOpacity(.8),
                        fontSize: 16.0),
                    onChanged: (query) => updateSearchQuery(query),
                  ),
                ),
              ),
            ],
          ),
          bottom: isSearching ? null : widget.appBarBottom,
        ),
        body: Stack(
          children: [
            widget.scaffoldBody,
            isSearching
                ? Container(
                    width: MediaQuery.of(context).size.width,
                    height: MediaQuery.of(context).size.height,
                    color: widget.searchBodyBackgroundColor,
                    child: StreamBuilder<List>(
                        stream: FirestoreServicePackage(
                                collectionName: widget.firestoreCollectionName,
                                searchBy: widget.searchBy ?? '',
                                dataListFromSnapshot:
                                    widget.dataListFromSnapshot,
                                limitOfRetrievedData:
                                    widget.limitOfRetrievedData)
                            .searchData(searchQuery),
                        builder: widget.builder),
                  )
                : SizedBox(
                    height: 0.0,
                    width: 0.0,
                  ),
          ],
        ));
  }

  void updateSearchQuery(String newQuery) {
    setState(() {
      searchQuery = newQuery;
    });
  }

  void stopSearching() {
    clearSearchQuery();
    setState(() {
      isSearching = false;
    });
  }

  void clearSearchQuery() {
    setState(() {
      searchQueryController.clear();
      updateSearchQuery("");
    });
  }

  @override
  void dispose() {
    // TODO: implement dispose
    searchQueryController.dispose();
    super.dispose();
  }
}
